@@ -108,6 +108,7 @@ struct _ColorSelectionPrivate
   guint changing : 1;
   guint default_set : 1;
   guint default_alpha_set : 1;
+  guint has_grab : 1;
   
   gdouble color[COLORSEL_NUM_CHANNELS];
   gdouble old_color[COLORSEL_NUM_CHANNELS];
@@ -140,15 +141,12 @@ struct _ColorSelectionPrivate
   /* Window for grabbing on */
   GtkWidget *dropper_grab_widget;
   guint32    grab_time;
-  gboolean   has_grab;
 
   /* Connection to settings */
   gulong settings_connection;
 };
 
 
-static void gtk_color_selection_init		(GtkColorSelection	 *colorsel);
-static void gtk_color_selection_class_init	(GtkColorSelectionClass	 *klass);
 static void gtk_color_selection_destroy		(GtkObject		 *object);
 static void gtk_color_selection_finalize        (GObject		 *object);
 static void update_color			(GtkColorSelection	 *colorsel);
@@ -183,7 +181,6 @@ static void     make_control_relations                  (AtkObject         *atk_
 static void     make_all_relations                      (AtkObject         *atk_obj,
                                                          ColorSelectionPrivate *priv);
 
-static gpointer parent_class = NULL;
 static guint color_selection_signals[LAST_SIGNAL] = { 0 };
 
 static const gchar default_colors[] = "black:white:gray50:red:purple:blue:light blue:green:yellow:orange:lavender:brown:goldenrod4:dodger blue:pink:light green:gray10:gray30:gray75:gray90";
@@ -297,7 +294,7 @@ color_sample_drag_end (GtkWidget      *widget,
 		       GdkDragContext *context,
 		       gpointer        data)
 {
-  g_object_set_data (G_OBJECT (widget), "gtk-color-selection-drag-window", NULL);
+  g_object_set_data (G_OBJECT (widget), I_("gtk-color-selection-drag-window"), NULL);
 }
 
 static void
@@ -374,7 +371,7 @@ color_sample_drag_handle (GtkWidget        *widget,
   vals[3] = priv->has_opacity ? colsrc[COLORSEL_OPACITY] * 0xffff : 0xffff;
   
   gtk_selection_data_set (selection_data,
-			  gdk_atom_intern ("application/x-color", FALSE),
+			  gdk_atom_intern_static_string ("application/x-color"),
 			  16, (guchar *)vals, 8);
 }
 
@@ -626,10 +623,6 @@ palette_paint (GtkWidget    *drawing_area,
   if (GTK_WIDGET_HAS_FOCUS (drawing_area))
     {
       set_focus_line_attributes (drawing_area, cr, &focus_width);
-      g_print ("%g %g %g %g\n",
-	       focus_width / 2., focus_width / 2.,
-	       (double)drawing_area->allocation.width - focus_width,
-	       (double)drawing_area->allocation.height - focus_width);
 
       cairo_rectangle (cr,
 		       focus_width / 2., focus_width / 2.,
@@ -723,7 +716,7 @@ palette_drag_handle (GtkWidget        *widget,
   vals[3] = 0xffff;
   
   gtk_selection_data_set (selection_data,
-			  gdk_atom_intern ("application/x-color", FALSE),
+			  gdk_atom_intern_static_string ("application/x-color"),
 			  16, (guchar *)vals, 8);
 }
 
@@ -732,7 +725,7 @@ palette_drag_end (GtkWidget      *widget,
 		  GdkDragContext *context,
 		  gpointer        data)
 {
-  g_object_set_data (G_OBJECT (widget), "gtk-color-selection-drag-window", NULL);
+  g_object_set_data (G_OBJECT (widget), I_("gtk-color-selection-drag-window"), NULL);
 }
 
 static GdkColor *
@@ -871,7 +864,7 @@ palette_set_color (GtkWidget         *drawing_area,
 			G_CALLBACK (palette_drag_handle),
 			colorsel);
       
-      g_object_set_data (G_OBJECT (drawing_area), "color_set",
+      g_object_set_data (G_OBJECT (drawing_area), I_("color_set"),
 			 GINT_TO_POINTER (1));
     }
 
@@ -880,7 +873,7 @@ palette_set_color (GtkWidget         *drawing_area,
   new_color[2] = color[2];
   new_color[3] = 1.0;
   
-  g_object_set_data_full (G_OBJECT (drawing_area), "color_val", new_color, (GDestroyNotify)g_free);
+  g_object_set_data_full (G_OBJECT (drawing_area), I_("color_val"), new_color, (GDestroyNotify)g_free);
 }
 
 static gboolean
@@ -953,7 +946,7 @@ do_popup (GtkColorSelection *colorsel,
   GtkWidget *mi;
   
   g_object_set_data (G_OBJECT (drawing_area),
-                     "gtk-color-sel",
+                     I_("gtk-color-sel"),
                      colorsel);
   
   menu = gtk_menu_new ();
@@ -980,7 +973,7 @@ palette_enter (GtkWidget        *drawing_area,
 	       gpointer        data)
 {
   g_object_set_data (G_OBJECT (drawing_area),
-		     "gtk-colorsel-have-pointer",
+		     I_("gtk-colorsel-have-pointer"),
 		     GUINT_TO_POINTER (TRUE));
 
   return FALSE;
@@ -992,7 +985,7 @@ palette_leave (GtkWidget        *drawing_area,
 	       gpointer        data)
 {
   g_object_set_data (G_OBJECT (drawing_area),
-		     "gtk-colorsel-have-pointer",
+		     I_("gtk-colorsel-have-pointer"),
 		     NULL);
 
   return FALSE;
@@ -1127,7 +1120,7 @@ palette_new (GtkColorSelection *colorsel)
 
   GTK_WIDGET_SET_FLAGS (retval, GTK_CAN_FOCUS);
   
-  g_object_set_data (G_OBJECT (retval), "color_set", GINT_TO_POINTER (0)); 
+  g_object_set_data (G_OBJECT (retval), I_("color_set"), GINT_TO_POINTER (0)); 
   gtk_widget_set_events (retval, GDK_BUTTON_PRESS_MASK
                          | GDK_BUTTON_RELEASE_MASK
                          | GDK_EXPOSURE_MASK
@@ -1227,6 +1220,17 @@ grab_color_at_mouse (GdkScreen *screen,
   priv = colorsel->private_data;
   
   image = gdk_drawable_get_image (root_window, x_root, y_root, 1, 1);
+  if (!image)
+    {
+      gint x, y;
+      GdkDisplay *display = gdk_screen_get_display (screen);
+      GdkWindow *window = gdk_display_get_window_at_pointer (display, &x, &y);
+      if (!window)
+	return;
+      image = gdk_drawable_get_image (window, x, y, 1, 1);
+      if (!image)
+	return;
+    }
   pixel = gdk_image_get_pixel (image, 0, 0);
   g_object_unref (image);
 
@@ -1289,9 +1293,7 @@ mouse_release (GtkWidget      *invisible,
 	       GdkEventButton *event,
 	       gpointer        data)
 {
-  GtkColorSelection *colorsel = data;
-  ColorSelectionPrivate *priv;
-  priv = colorsel->private_data;  
+  /* GtkColorSelection *colorsel = data; */
 
   if (event->button != 1)
     return FALSE;
@@ -1393,9 +1395,7 @@ mouse_press (GtkWidget      *invisible,
 	     GdkEventButton *event,
 	     gpointer        data)
 {
-  GtkColorSelection *colorsel = data;
-  ColorSelectionPrivate *priv;
-  priv = colorsel->private_data;
+  /* GtkColorSelection *colorsel = data; */
   
   if (event->type == GDK_BUTTON_PRESS &&
       event->button == 1)
@@ -1427,20 +1427,31 @@ get_screen_color (GtkWidget *button)
   GdkScreen *screen = gtk_widget_get_screen (GTK_WIDGET (button));
   GdkCursor *picker_cursor;
   GdkGrabStatus grab_status;
+  GtkWidget *grab_widget, *toplevel;
+
   guint32 time = gtk_get_current_event_time ();
   
   if (priv->dropper_grab_widget == NULL)
     {
-      priv->dropper_grab_widget = gtk_invisible_new_for_screen (screen);
+      grab_widget = gtk_window_new (GTK_WINDOW_POPUP);
+      gtk_window_set_screen (GTK_WINDOW (grab_widget), screen);
+      gtk_window_resize (GTK_WINDOW (grab_widget), 1, 1);
+      gtk_window_move (GTK_WINDOW (grab_widget), -100, -100);
+      gtk_widget_show (grab_widget);
 
-      gtk_widget_add_events (priv->dropper_grab_widget,
+      gtk_widget_add_events (grab_widget,
                              GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK);
       
-      gtk_widget_show (priv->dropper_grab_widget);
+      toplevel = gtk_widget_get_toplevel (colorsel);
+  
+      if (GTK_IS_WINDOW (toplevel))
+	{
+	  if (GTK_WINDOW (toplevel)->group)
+	    gtk_window_group_add_window (GTK_WINDOW (toplevel)->group, 
+					 GTK_WINDOW (grab_widget));
+	}
 
-      gdk_window_set_user_data (priv->dropper_grab_widget->window, colorsel);
-      gdk_window_reparent (priv->dropper_grab_widget->window, 
-			   GTK_WIDGET (colorsel)->window, 0, 0);
+      priv->dropper_grab_widget = grab_widget;
     }
 
   if (gdk_keyboard_grab (priv->dropper_grab_widget->window,
@@ -1644,7 +1655,7 @@ make_label_spinbutton (GtkColorSelection *colorsel,
     {
       adjust = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 255.0, 1.0, 1.0, 1.0));
     }
-  g_object_set_data (G_OBJECT (adjust), "COLORSEL", colorsel);
+  g_object_set_data (G_OBJECT (adjust), I_("COLORSEL"), colorsel);
   *spinbutton = gtk_spin_button_new (adjust, 10.0, 0);
 
   gtk_tooltips_set_tip (priv->tooltips, *spinbutton, tooltip, NULL);  
@@ -1655,7 +1666,7 @@ make_label_spinbutton (GtkColorSelection *colorsel,
   label = gtk_label_new_with_mnemonic (text);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), *spinbutton);
 
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_table_attach_defaults (GTK_TABLE (table), label, i, i+1, j, j+1);
   gtk_table_attach_defaults (GTK_TABLE (table), *spinbutton, i+1, i+2, j, j+1);
 }
@@ -1816,33 +1827,7 @@ default_change_palette_func (GdkScreen	    *screen,
   g_free (str);
 }
 
-GType
-gtk_color_selection_get_type (void)
-{
-  static GType color_selection_type = 0;
-  
-  if (!color_selection_type)
-    {
-      static const GTypeInfo color_selection_info =
-      {
-        sizeof (GtkColorSelectionClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-        (GClassInitFunc) gtk_color_selection_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-        sizeof (GtkColorSelection),
-	0,		/* n_preallocs */
-        (GInstanceInitFunc) gtk_color_selection_init,
-      };
-      
-      color_selection_type =
-	g_type_register_static (GTK_TYPE_VBOX, "GtkColorSelection",
-				&color_selection_info, 0);
-    }
-  
-  return color_selection_type;
-}
+G_DEFINE_TYPE (GtkColorSelection, gtk_color_selection, GTK_TYPE_VBOX)
 
 static void
 gtk_color_selection_class_init (GtkColorSelectionClass *klass)
@@ -1855,8 +1840,6 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
   object_class = GTK_OBJECT_CLASS (klass);
   widget_class = GTK_WIDGET_CLASS (klass);
   
-  parent_class = g_type_class_peek_parent (klass);
-  
   object_class->destroy = gtk_color_selection_destroy;
   gobject_class->finalize = gtk_color_selection_finalize;
   
@@ -1898,7 +1881,7 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
 						      GTK_PARAM_READWRITE));
   
   color_selection_signals[COLOR_CHANGED] =
-    g_signal_new ("color_changed",
+    g_signal_new (I_("color_changed"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkColorSelectionClass, color_changed),
@@ -1911,6 +1894,8 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
                                                       P_("Palette to use in the color selector"),
                                                       default_colors,
                                                       GTK_PARAM_READWRITE));
+
+   g_type_class_add_private (gobject_class, sizeof (ColorSelectionPrivate));
 }
 
 /* widget functions */
@@ -1926,23 +1911,22 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   gint i, j;
   ColorSelectionPrivate *priv;
   AtkObject *atk_obj;
+  GList *focus_chain = NULL;
   
   gtk_widget_push_composite_child ();
 
-  priv = colorsel->private_data = g_new0 (ColorSelectionPrivate, 1);
+  priv = colorsel->private_data = G_TYPE_INSTANCE_GET_PRIVATE (colorsel, GTK_TYPE_COLOR_SELECTION, ColorSelectionPrivate);
   priv->changing = FALSE;
   priv->default_set = FALSE;
   priv->default_alpha_set = FALSE;
   
   priv->tooltips = gtk_tooltips_new ();
-  g_object_ref (priv->tooltips);
-  gtk_object_sink (GTK_OBJECT (priv->tooltips));
+  g_object_ref_sink (priv->tooltips);
   
-  gtk_box_set_spacing (GTK_BOX (colorsel), 4);
-  top_hbox = gtk_hbox_new (FALSE, 8);
+  top_hbox = gtk_hbox_new (FALSE, 12);
   gtk_box_pack_start (GTK_BOX (colorsel), top_hbox, FALSE, FALSE, 0);
   
-  vbox = gtk_vbox_new (FALSE, 4);
+  vbox = gtk_vbox_new (FALSE, 6);
   priv->triangle_colorsel = gtk_hsv_new ();
   g_signal_connect (priv->triangle_colorsel, "changed",
                     G_CALLBACK (hsv_changed), colorsel);
@@ -1952,7 +1936,7 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   gtk_tooltips_set_tip (priv->tooltips, priv->triangle_colorsel,
                         _("Select the color you want from the outer ring. Select the darkness or lightness of that color using the inner triangle."), NULL);
   
-  hbox = gtk_hbox_new (FALSE, 4);
+  hbox = gtk_hbox_new (FALSE, 6);
   gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   
   frame = gtk_frame_new (NULL);
@@ -1965,7 +1949,7 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   button = gtk_button_new ();
 
   gtk_widget_set_events (button, GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
-  g_object_set_data (G_OBJECT (button), "COLORSEL", colorsel); 
+  g_object_set_data (G_OBJECT (button), I_("COLORSEL"), colorsel); 
   g_signal_connect (button, "clicked",
                     G_CALLBACK (get_screen_color), NULL);
   picker_image = gtk_image_new_from_stock (GTK_STOCK_COLOR_PICKER, GTK_ICON_SIZE_BUTTON);
@@ -1977,12 +1961,12 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
                         button,
                         _("Click the eyedropper, then click a color anywhere on your screen to select that color."), NULL);
   
-  top_right_vbox = gtk_vbox_new (FALSE, 4);
+  top_right_vbox = gtk_vbox_new (FALSE, 6);
   gtk_box_pack_start (GTK_BOX (top_hbox), top_right_vbox, FALSE, FALSE, 0);
   table = gtk_table_new (8, 6, FALSE);
   gtk_box_pack_start (GTK_BOX (top_right_vbox), table, FALSE, FALSE, 0);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 12);
   
   make_label_spinbutton (colorsel, &priv->hue_spinbutton, _("_Hue:"), table, 0, 0, COLORSEL_HUE,
                          _("Position on the color wheel."));
@@ -1998,12 +1982,12 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   make_label_spinbutton (colorsel, &priv->blue_spinbutton, _("_Blue:"), table, 6, 2, COLORSEL_BLUE,
                          _("Amount of blue light in the color."));
   gtk_table_attach_defaults (GTK_TABLE (table), gtk_hseparator_new (), 0, 8, 3, 4); 
-  
-  priv->opacity_label = gtk_label_new_with_mnemonic (_("_Opacity:")); 
-  gtk_misc_set_alignment (GTK_MISC (priv->opacity_label), 1.0, 0.5); 
+
+  priv->opacity_label = gtk_label_new_with_mnemonic (_("Op_acity:")); 
+  gtk_misc_set_alignment (GTK_MISC (priv->opacity_label), 0.0, 0.5); 
   gtk_table_attach_defaults (GTK_TABLE (table), priv->opacity_label, 0, 1, 4, 5); 
   adjust = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 255.0, 1.0, 1.0, 0.0)); 
-  g_object_set_data (G_OBJECT (adjust), "COLORSEL", colorsel); 
+  g_object_set_data (G_OBJECT (adjust), I_("COLORSEL"), colorsel); 
   priv->opacity_slider = gtk_hscale_new (adjust);
   gtk_tooltips_set_tip (priv->tooltips,
                         priv->opacity_slider,
@@ -2025,9 +2009,9 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
                     G_CALLBACK (opacity_entry_changed), colorsel);
   gtk_table_attach_defaults (GTK_TABLE (table), priv->opacity_entry, 7, 8, 4, 5);
   
-  label = gtk_label_new_with_mnemonic (_("Color _Name:"));
+  label = gtk_label_new_with_mnemonic (_("Color _name:"));
   gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 5, 6);
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   priv->hex_entry = gtk_entry_new ();
 
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), priv->hex_entry);
@@ -2043,9 +2027,20 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
                         _("You can enter an HTML-style hexadecimal color value, or simply a color name such as 'orange' in this entry."), NULL);
   
   gtk_entry_set_width_chars (GTK_ENTRY (priv->hex_entry), 7);
-  gtk_table_set_col_spacing (GTK_TABLE (table), 3, 15);
   gtk_table_attach_defaults (GTK_TABLE (table), priv->hex_entry, 1, 5, 5, 6);
-  
+
+  focus_chain = g_list_append (focus_chain, priv->hue_spinbutton);
+  focus_chain = g_list_append (focus_chain, priv->sat_spinbutton);
+  focus_chain = g_list_append (focus_chain, priv->val_spinbutton);
+  focus_chain = g_list_append (focus_chain, priv->red_spinbutton);
+  focus_chain = g_list_append (focus_chain, priv->green_spinbutton);
+  focus_chain = g_list_append (focus_chain, priv->blue_spinbutton);
+  focus_chain = g_list_append (focus_chain, priv->opacity_slider);
+  focus_chain = g_list_append (focus_chain, priv->opacity_entry);
+  focus_chain = g_list_append (focus_chain, priv->hex_entry);
+  gtk_container_set_focus_chain (GTK_CONTAINER (table), focus_chain);
+  g_list_free (focus_chain);
+
   /* Set up the palette */
   table = gtk_table_new (GTK_CUSTOM_PALETTE_HEIGHT, GTK_CUSTOM_PALETTE_WIDTH, TRUE);
   gtk_table_set_row_spacings (GTK_TABLE (table), 1);
@@ -2058,19 +2053,16 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
 	}
     }
   set_selected_palette (colorsel, 0, 0);
-  priv->palette_frame = gtk_frame_new (NULL);
-  label = gtk_label_new_with_mnemonic (_("_Palette"));
-  gtk_frame_set_label_widget (GTK_FRAME (priv->palette_frame), label);
+  priv->palette_frame = gtk_vbox_new (FALSE, 6);
+  label = gtk_label_new_with_mnemonic (_("_Palette:"));
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_box_pack_start (GTK_BOX (priv->palette_frame), label, FALSE, FALSE, 0);
 
   gtk_label_set_mnemonic_widget (GTK_LABEL (label),
                                  priv->custom_palette[0][0]);
   
   gtk_box_pack_end (GTK_BOX (top_right_vbox), priv->palette_frame, FALSE, FALSE, 0);
-  vbox = gtk_vbox_new (FALSE, 4);
-  gtk_container_add (GTK_CONTAINER (priv->palette_frame), vbox);
-  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
-
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
+  gtk_box_pack_start (GTK_BOX (priv->palette_frame), table, FALSE, FALSE, 0);
   
   gtk_widget_show_all (top_hbox);
 
@@ -2117,21 +2109,13 @@ gtk_color_selection_destroy (GtkObject *object)
       priv->tooltips = NULL;
     }
   
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);
+  GTK_OBJECT_CLASS (gtk_color_selection_parent_class)->destroy (object);
 }
 
 static void
 gtk_color_selection_finalize (GObject *object)
 {
-  GtkColorSelection *cselection = GTK_COLOR_SELECTION (object);
-  
-  if (cselection->private_data)
-    {
-      g_free (cselection->private_data);
-      cselection->private_data = NULL;
-    }
-  
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_color_selection_parent_class)->finalize (object);
 }
 
 static void
@@ -2147,7 +2131,7 @@ gtk_color_selection_realize (GtkWidget *widget)
 						 widget);
   update_palette (colorsel);
 
-  GTK_WIDGET_CLASS (parent_class)->realize (widget);
+  GTK_WIDGET_CLASS (gtk_color_selection_parent_class)->realize (widget);
 }
 
 static void
@@ -2159,7 +2143,7 @@ gtk_color_selection_unrealize (GtkWidget *widget)
 
   g_signal_handler_disconnect (settings, priv->settings_connection);
 
-  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+  GTK_WIDGET_CLASS (gtk_color_selection_parent_class)->unrealize (widget);
 }
 
 /* We override show-all since we have internal widgets that
@@ -2335,7 +2319,6 @@ gtk_color_selection_set_current_color (GtkColorSelection *colorsel,
 {
   ColorSelectionPrivate *priv;
   gint i;
-  gboolean previous_changed;
   
   g_return_if_fail (GTK_IS_COLOR_SELECTION (colorsel));
   g_return_if_fail (color != NULL);
@@ -2353,7 +2336,6 @@ gtk_color_selection_set_current_color (GtkColorSelection *colorsel,
 		  &priv->color[COLORSEL_VALUE]);
   if (priv->default_set == FALSE)
     {
-      previous_changed = TRUE;
       for (i = 0; i < COLORSEL_NUM_CHANNELS; i++)
 	priv->old_color[i] = priv->color[i];
     }
